/*
 * Copyright 2000 Murray Cumming
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef APP_WITHDOC_H
#define APP_WITHDOC_H

#include "App.h"
#include "../Document/Document.h"

namespace Bakery
{

//Features:
//- Derive from this class
//- 1 document per application instance. Uses Document-derived class polymorphically.
//- Override pure virtual method to create new blank document.
//- Appropriate Default handling of document open, save, save as.
//- Appropriate checking of document 'modified' status - asks user about unsaved changes.
//- Asks user about overwriting existing documents.
//- Override methods to add/change menus/toolbars/statusbar.
//  - Default is basic File, Edit, Help menus and toolbar icons.
//- Shows document name (or 'untitled') in window title.
//- Shows * in title bar for unsaved docs. Overridable to e.g. shade a Save icon.

//TODO:
//- Enforce a file extension.
//- Recent documents - when GConf is ready.
//- Bonobo integration - when Bonobo is ready. (An App subclass?)
//- Printing - GnomePrint?
//  - Print Setup
//  - Print Preview
//- Multiple document-types:
//  - File/New sub menu
//  - Some way to associate a view with a document type
//  Gnome::MDI App.

class App_WithDoc : public App
{
public: 
	App_WithDoc(const string& appname, const string& title); //Don't forget to call init() too.
	virtual ~App_WithDoc();
	
protected:
  virtual void init(); //overriden to create document.
  virtual void init_menus_file(); //overridden to add open/save/save as.
  virtual void init_toolbars(); //overridden to add open/save
	virtual void init_create_document(); //override this to new() the specific document type.

  virtual Document* get_document(); //static_cast<> or dynamic_cast<> this pointer to the correct type.

  //Signal handlers:

  //Menu items:
  virtual void on_menu_File_Open();
  virtual void on_menu_File_SaveAs();
  virtual void on_menu_File_Save();
  virtual void on_menu_File_Close();

  //FileSelections (Open, Save):
  virtual void on_FileSelection_Open_OK();
  virtual void on_FileSelection_Open_Cancel();
  virtual void on_FileSelection_Save_OK();
  virtual void on_FileSelection_Save_Cancel();

  //Document:
  virtual void on_Document_modified(); //Update visual status.
  virtual void on_Document_load(); //override this to show document contents.

  virtual void offer_to_save_changes();
  virtual void update_window_title();

  //Document:
  Document* m_pDocument; //An instance of a derived type.
  bool m_bCloseAfterSave;

  //Member widets:
  Gtk::FileSelection m_FileSelection_Open, m_FileSelection_Save;
};

} //namespace

#endif
